@@ -134,7 +134,7 @@ class LinuxSCSI(executor.Executor):
         LOG.debug("remove multipath device %s", device)
         mpath_dev = self.find_multipath_device(device)
         if mpath_dev:
-            self.flush_multipath_device(mpath_dev['id'])
+            self.flush_multipath_device(mpath_dev['name'])
             devices = mpath_dev['devices']
             LOG.debug("multipath LUNs to remove %s", devices)
             for device in devices:
@@ -150,12 +150,16 @@ class LinuxSCSI(executor.Executor):
             LOG.warning(_LW("Failed to flush IO buffers prior to removing "
                             "device: %(code)s"), {'code': exc.exit_code})
 
+    @utils.retry(exceptions=putils.ProcessExecutionError)
     def flush_multipath_device(self, device):
         try:
             LOG.debug("Flush multipath device %s", device)
             self._execute('multipath', '-f', device, run_as_root=True,
                           root_helper=self._root_helper)
         except putils.ProcessExecutionError as exc:
+            if exc.exit_code == 1 and 'map in use' in exc.stdout:
+                LOG.debug('Multipath is in use, cannot be flushed yet.')
+                raise
             LOG.warning(_LW("multipath call failed exit %(code)s"),
                         {'code': exc.exit_code})
 
@@ -433,7 +437,7 @@ class LinuxSCSI(executor.Executor):
 
     def _format_lun_id(self, lun_id):
         if lun_id < 256:
-                return lun_id
+            return lun_id
         else:
             return ("0x%04x%04x00000000" %
                     (lun_id & 0xffff, lun_id >> 16 & 0xffff))
