@@ -13,42 +13,43 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
+import binascii
 import mock
+import uuid
 
+
+from castellan.common.objects import symmetric_key as key
 from os_brick.encryptors import luks
 from os_brick.tests.encryptors import test_cryptsetup
 from oslo_concurrency import processutils as putils
 
 
-@mock.patch('os_brick.executor.encodeutils.safe_decode',
-            lambda x, errors=None: x)
 class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
-    def _create(self, root_helper, connection_info, keymgr, execute):
-        return luks.LuksEncryptor(root_helper=root_helper,
-                                  connection_info=connection_info,
-                                  keymgr=keymgr,
-                                  execute=execute)
+    def _create(self):
+        return luks.LuksEncryptor(root_helper=self.root_helper,
+                                  connection_info=self.connection_info,
+                                  keymgr=self.keymgr)
 
-    def test_is_luks(self):
-        luks.is_luks(self.root_helper, self.dev_path)
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_is_luks(self, mock_execute):
+        luks.is_luks(self.root_helper, self.dev_path, execute=mock_execute)
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
                       run_as_root=True, root_helper=self.root_helper,
                       check_exit_code=True),
         ], any_order=False)
 
+    @mock.patch('os_brick.executor.Executor._execute')
     @mock.patch('os_brick.encryptors.luks.LOG')
-    def test_is_luks_with_error(self, mock_log):
+    def test_is_luks_with_error(self, mock_log, mock_execute):
         error_msg = "Device %s is not a valid LUKS device." % self.dev_path
-        self.mock_execute.side_effect = \
-            putils.ProcessExecutionError(exit_code=1,
-                                         stderr=error_msg)
+        mock_execute.side_effect = putils.ProcessExecutionError(
+            exit_code=1, stderr=error_msg)
 
-        luks.is_luks(self.root_helper, self.dev_path)
+        luks.is_luks(self.root_helper, self.dev_path, execute=mock_execute)
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
                       run_as_root=True, root_helper=self.root_helper,
                       check_exit_code=True),
@@ -56,19 +57,11 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
 
         self.assertEqual(1, mock_log.warning.call_count)  # warning logged
 
-    def test_is_luks_with_execute(self):
-        mock_execute = mock.Mock()
-        luks.is_luks(self.root_helper, self.dev_path, execute=mock_execute)
-        self.assertListEqual(
-            [mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
-                       run_as_root=True, root_helper=self.root_helper,
-                       check_exit_code=True)],
-            mock_execute.call_args_list)
-
-    def test__format_volume(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test__format_volume(self, mock_execute):
         self.encryptor._format_volume("passphrase")
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', '--batch-mode', 'luksFormat',
                       '--key-file=-', self.dev_path,
                       process_input='passphrase',
@@ -76,26 +69,29 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       run_as_root=True, check_exit_code=True, attempts=3),
         ])
 
-    def test__open_volume(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test__open_volume(self, mock_execute):
         self.encryptor._open_volume("passphrase")
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
                       self.dev_name, process_input='passphrase',
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
         ])
 
-    def test_attach_volume(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_attach_volume(self, mock_execute):
+        fake_key = uuid.uuid4().hex
         self.encryptor._get_key = mock.MagicMock()
         self.encryptor._get_key.return_value = (
-            test_cryptsetup.fake__get_key(None))
+            test_cryptsetup.fake__get_key(None, fake_key))
 
         self.encryptor.attach_volume(None)
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input='0' * 32,
+                      self.dev_name, process_input=fake_key,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
             mock.call('ln', '--symbolic', '--force',
@@ -104,12 +100,14 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       run_as_root=True, check_exit_code=True),
         ])
 
-    def test_attach_volume_not_formatted(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_attach_volume_not_formatted(self, mock_execute):
+        fake_key = uuid.uuid4().hex
         self.encryptor._get_key = mock.MagicMock()
         self.encryptor._get_key.return_value = (
-            test_cryptsetup.fake__get_key(None))
+            test_cryptsetup.fake__get_key(None, fake_key))
 
-        self.mock_execute.side_effect = [
+        mock_execute.side_effect = [
             putils.ProcessExecutionError(exit_code=1),  # luksOpen
             putils.ProcessExecutionError(exit_code=1),  # isLuks
             mock.DEFAULT,  # luksFormat
@@ -119,20 +117,20 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
 
         self.encryptor.attach_volume(None)
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input='0' * 32,
+                      self.dev_name, process_input=fake_key,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
             mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
             mock.call('cryptsetup', '--batch-mode', 'luksFormat',
-                      '--key-file=-', self.dev_path, process_input='0' * 32,
+                      '--key-file=-', self.dev_path, process_input=fake_key,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True, attempts=3),
             mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input='0' * 32,
+                      self.dev_name, process_input=fake_key,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
             mock.call('ln', '--symbolic', '--force',
@@ -141,12 +139,14 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       run_as_root=True, check_exit_code=True),
         ], any_order=False)
 
-    def test_attach_volume_fail(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_attach_volume_fail(self, mock_execute):
+        fake_key = uuid.uuid4().hex
         self.encryptor._get_key = mock.MagicMock()
         self.encryptor._get_key.return_value = (
-            test_cryptsetup.fake__get_key(None))
+            test_cryptsetup.fake__get_key(None, fake_key))
 
-        self.mock_execute.side_effect = [
+        mock_execute.side_effect = [
             putils.ProcessExecutionError(exit_code=1),  # luksOpen
             mock.DEFAULT,  # isLuks
         ]
@@ -154,9 +154,9 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
         self.assertRaises(putils.ProcessExecutionError,
                           self.encryptor.attach_volume, None)
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
-                      self.dev_name, process_input='0' * 32,
+                      self.dev_name, process_input=fake_key,
                       root_helper=self.root_helper,
                       run_as_root=True, check_exit_code=True),
             mock.call('cryptsetup', 'isLuks', '--verbose', self.dev_path,
@@ -164,20 +164,93 @@ class LuksEncryptorTestCase(test_cryptsetup.CryptsetupEncryptorTestCase):
                       run_as_root=True, check_exit_code=True),
         ], any_order=False)
 
-    def test__close_volume(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test__close_volume(self, mock_execute):
         self.encryptor.detach_volume()
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'luksClose', self.dev_name,
                       root_helper=self.root_helper,
-                      attempts=3, run_as_root=True, check_exit_code=True),
+                      attempts=3, run_as_root=True, check_exit_code=[0, 4]),
         ])
 
-    def test_detach_volume(self):
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_detach_volume(self, mock_execute):
         self.encryptor.detach_volume()
 
-        self.assert_exec_has_calls([
+        mock_execute.assert_has_calls([
             mock.call('cryptsetup', 'luksClose', self.dev_name,
                       root_helper=self.root_helper,
-                      attempts=3, run_as_root=True, check_exit_code=True),
+                      attempts=3, run_as_root=True, check_exit_code=[0, 4]),
         ])
+
+    def test_get_mangled_passphrase(self):
+        # Confirm that a mangled passphrase is provided as per bug#1633518
+        unmangled_raw_key = bytes(binascii.unhexlify('0725230b'))
+        symmetric_key = key.SymmetricKey('AES', len(unmangled_raw_key) * 8,
+                                         unmangled_raw_key)
+        unmangled_encoded_key = symmetric_key.get_encoded()
+        self.assertEqual(self.encryptor._get_mangled_passphrase(
+            unmangled_encoded_key), '72523b')
+
+    @mock.patch('os_brick.executor.Executor._execute')
+    def test_attach_volume_unmangle_passphrase(self, mock_execute):
+        fake_key = '0725230b'
+        fake_key_mangled = '72523b'
+        self.encryptor._get_key = mock.MagicMock()
+        self.encryptor._get_key.return_value = \
+            test_cryptsetup.fake__get_key(None, fake_key)
+
+        mock_execute.side_effect = [
+            putils.ProcessExecutionError(exit_code=2),  # luksOpen
+            mock.DEFAULT,  # luksOpen
+            mock.DEFAULT,  # luksClose
+            mock.DEFAULT,  # luksAddKey
+            mock.DEFAULT,  # luksOpen
+            mock.DEFAULT,  # luksClose
+            mock.DEFAULT,  # luksRemoveKey
+            mock.DEFAULT,  # luksOpen
+            mock.DEFAULT,  # ln
+        ]
+
+        self.encryptor.attach_volume(None)
+
+        mock_execute.assert_has_calls([
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key_mangled,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+            mock.call('cryptsetup', 'luksClose', self.dev_name,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=[0, 4], attempts=3),
+            mock.call('cryptsetup', 'luksAddKey', self.dev_path,
+                      process_input=''.join([fake_key_mangled,
+                                             '\n', fake_key,
+                                             '\n', fake_key]),
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+            mock.call('cryptsetup', 'luksClose', self.dev_name,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=[0, 4], attempts=3),
+            mock.call('cryptsetup', 'luksRemoveKey', self.dev_path,
+                      process_input=fake_key_mangled,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+            mock.call('cryptsetup', 'luksOpen', '--key-file=-', self.dev_path,
+                      self.dev_name, process_input=fake_key,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+            mock.call('ln', '--symbolic', '--force',
+                      '/dev/mapper/%s' % self.dev_name, self.symlink_path,
+                      root_helper=self.root_helper, run_as_root=True,
+                      check_exit_code=True),
+        ], any_order=False)
+        self.assertEqual(9, mock_execute.call_count)
